# -*- coding: utf8 -*-

# Copyright 2024 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import scipy.linalg as spla

class EigenSolver:
    """Eigen solver
    """
    def __init__(self, l, n, g=0):
        self._l = l # reference length
        self._n = n # number of modes
        self._g = (1 + 1j * g) # structural damping (complex proportional stiffness)

    def compute_natural(self, m, k):
        """Compute natural frequencies and mode shapes
        m : mass matrix
        k : stiffness matrix
        """
        l, v = spla.eig(k, m) # det(-omega^2 * M + K) = 0
        return np.sqrt(np.real(l)), v

    def compute(self, rho, u, m, k, q, v0=None):
        """Compute eigenvalues and eigenvectors
        rho : fuid density
        u : freestream velocity
        m : mass matrix
        k : stiffness matrix
        q : aerodynamic matrix
        v0 : converged mode shapes from previous iteration (airspeed)
        """
        p, v = spla.eig(self._g * k - 0.5 * rho * u**2 * q, -u**2 / self._l**2 * m) # det((p*u/l)^2*M + K - q_inf*Q) = 0
        p = np.sqrt(p)
        # Sort
        p = np.where(np.imag(p) < 0, -p, p) # change sign of p if imag(p) < 0
        if v0 is None:
            srt = np.imag(p).argsort()
        else:
            srt = self._sort(v, v0)
        return p[srt].T, v[:,srt]

    def compute_gradients(self, rho, u, m, k, q, p, v, dm, dk, dq):
        """Compute the derivative of an eigenvalue problem
        rho : fuid density
        u : freestream velocity
        m : mass matrix
        k : stiffness matrix
        q : aerodynamic matrix
        p : eigenvalue solution
        v : eigenvector solution
        m : derivative of mass matrix
        k : derivative of stiffness matrix
        q : derivative of aerodynamic matrix
        """
        a = np.zeros((self._n + 1, self._n + 1), dtype=complex)
        b = np.zeros((self._n + 1, 1), dtype=complex)
        a[:-1,0] = -2 * p * u**2 / self._l**2 * m @ v
        a[:-1,1:] = p**2 * u**2 / self._l**2 * m + self._g * k - 0.5 * rho * u**2 * q
        a[-1,0] = 0
        a[-1,1:] = v.T
        b[:-1,0] = (p**2 * u**2 / self._l**2 * dm + self._g * dk - 0.5 * rho * u**2 * dq) @ v
        b[-1,0] = 0
        x = spla.lu_solve((spla.lu_factor(a)), b)
        return x[0,0]

    def _sort(self, v, v0):
        """Sort current modes according to their correlation to previous modes
        v : mode shapes from current iteration
        v0 : converged mode shapes from previous iteration
        """
        # Compute correlation matrix
        mac = np.zeros((self._n, self._n))
        for i in range(self._n):
            for j in range(self._n):
                mac[i,j] = np.linalg.norm(np.dot(v[:,i], np.conjugate(v0[:,j]))) / (np.linalg.norm(v[:,i]) * np.linalg.norm(v0[:,j]))
        # Find maximum for each mode
        return np.argmax(mac, axis=0)
