# -*- coding: utf8 -*-

# Copyright 2024 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .solution import Solution
import numpy as np
from scipy import interpolate

class Pk(Solution):
    """Standard p-k method for flutter solution
    """
    def __init__(self, name, flu, eig, agg, mach, k_ref, l_ref, vrb, n_modes, n_speed, tol=1e-3, max_it=10):
        super().__init__(name, flu, eig, agg, mach, k_ref, l_ref, vrb, n_modes, n_speed)
        self.name = 'PK' # solver name
        self._tol = tol # tolerance
        self._max_it = max_it # maximum number of iterations

    def compute(self):
        """Compute frequencies and dampings
        """
        omega_0, v_0 = self._eig.compute_natural(self._m, self._k) # natural frequencies and modes
        self.rho, self.speed = self._flu.compute(self._mach)
        for ivel in range(self.n_speed):
            # Compute eigenvalues at zero airspeed
            if self.speed[ivel] == 0:
                self.freq[ivel, :] = omega_0
                continue
            v_tmp = np.zeros((self.n_modes, self.n_modes), dtype=complex) # temporary container for modes
            for imod in range(self.n_modes):
                k = omega_0[imod] * self._lref / self.speed[ivel] # guess reduced frequency
                it = 0
                while True:
                    # Interpolate loads and solve eigenvalue problem
                    q = self.__interp(k, self._kref, self._q)
                    p, v = self._eig.compute(self.rho[ivel], self.speed[ivel], self._m, self._k, q, v_0)
                    p = p[imod]
                    # Check if reduced frequency is converged
                    if abs(np.imag(p) - k) < self._tol or it == self._max_it:
                        v_tmp[:, imod] = v[:, imod]
                        self.freq[ivel, imod] = np.imag(p) * self.speed[ivel] / self._lref / (2 * np.pi)
                        self.damp[ivel, imod] = np.real(p) / np.imag(p)
                        if it == self._max_it:
                            print(f'PK: could not converge for mode {imod} at velocity {self.speed[ivel]}, using k={np.imag(p)} with tolerance {abs(np.imag(p) - k)}.')
                        break
                    else:
                        k = np.imag(p)
                        it += 1
            # Update modes for mode tracking
            v_0 = v_tmp
        # Compute aggregated damping over modes then over dynamic pressures
        self._ksm = np.zeros(self.n_speed)
        for i in range(self.n_speed):
            self._ksm[i] = self._agg.compute(self.damp[i,:])
        self.damp_agg = self._agg.compute(self._ksm)

    def compute_gradients(self):
        """Compute the derivatives of the damping aggregations wrt mass, stiffness and aerodynamic terms
        """
        raise NotImplementedError()

    def __interp(self, k, k_ref, q_ref):
        """Match the frequency and interpolate the eigenvalues and eigenvectors
        k : interpolation reduced frequency
        k_ref : reference reduced frequencies
        q_ref : reference GAF matrices
        """
        f_re = interpolate.PchipInterpolator(k_ref, np.real(q_ref), axis=0, extrapolate=True)
        f_im = interpolate.PchipInterpolator(k_ref, np.imag(q_ref), axis=0, extrapolate=True)
        q_re = f_re(k)
        q_im = f_im(k)
        return q_re + 1j * q_im
