#!/usr/bin/env python3
# -*- coding: utf8 -*-
# test encoding: à-é-è-ô-ï-€

# Copyright 2024 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Compute flutter of AGARD 445.6 wing

from pypk import init_pypk
import numpy as np

def get_cfg():
    cfg = {
    'name': 'agard', # case name
    'k_ref': np.array([0.1, 0.3]), # reference reduced frequencies
    'l_ref': 0.5 * 0.559, # reference length (half root chord)
    'mach': 0.901, # freesteam Mach number
    'rho_inf': 0.0995, # freesteam density
    'u_idx': np.linspace(0.2, 0.5, 31, endpoint=True), # velocity index range
    'mu': 143.920, # mass ratio
    'f_ref': 40.3511, # frequency of first torsional mode
    'n_modes': 4, # number of modes
    'rho_ks': 1e6, # KS aggregation parameter for KS
    'method': 'nipk', # method type
    'fluid': 'unmatched', # fluid type
    'vrb': 1 # verbosity level
    }
    return cfg

def get_matrices():
    # Structural mass matrix
    m = np.diag([1.0, 1.0, 1.0, 1.0])
    # Structural stiffness matrix
    k = np.diag([3695.2, 63392.2, 99892.1, 368026.0])
    # Generalized aerodynamic force matrices (at k=0.1 and k=0.3)
    q = np.array([[[-7.747e-01-2.058e-01j, -6.014e+00-1.949e-01j,  3.962e+00+6.734e-02j, -3.403e+00-1.784e-01j],
                   [ 9.520e-01-1.703e-02j,  4.900e+00-1.707e+00j, -4.416e+00+1.048e+00j, -5.951e+00-3.470e-01j],
                   [-6.597e-02+6.036e-02j,  3.580e-01+5.907e-01j, -1.046e+00-6.893e-01j, -8.272e-02-6.273e-01j],
                   [-1.238e-01+8.540e-02j,  3.588e-01+5.434e-01j,  8.221e-01-4.957e-01j,  2.922e+00-9.594e-01j]],
                  [[-6.560e-01-6.408e-01j, -5.628e+00-1.028e+00j,  3.637e+00+5.146e-01j, -3.498e+00-6.270e-01j],
                   [ 9.918e-01-1.579e-02j,  3.545e+00-3.832e+00j, -3.405e+00+2.284e+00j, -5.881e+00-6.028e-01j],
                   [-3.243e-02+1.393e-01j,  9.077e-01+1.124e+00j, -1.434e+00-1.622e+00j, -6.827e-01-1.797e+00j],
                   [ 3.645e-03+2.359e-01j,  1.521e+00+7.309e-01j,  1.015e-01-9.865e-01j,  2.600e+00-3.367e+00j]]]
    )
    return m, k, q

def compute(sol):
    # Compute evolution of frequencies and dampings with airspeed
    sol.compute()
    sol.find_flutter()
    # Save results
    sol.save('_' + sol.name.lower())
    sol.plot('_' + sol.name.lower(), show=False, format='pdf')

def main():
    # Set up p-k and non-iterative p-k method
    cfg = get_cfg()
    nipk = init_pypk(cfg)
    cfg['method'] = 'pk'
    pk = init_pypk(cfg)
    m, k, q = get_matrices()
    nipk.set_matrices(m, k, q)
    pk.set_matrices(m, k, q)
    # Compute using the two methods
    compute(nipk)
    compute(pk)
    # Check
    if nipk.f_mode != 0:
        raise RuntimeError('Test failed: (Nipk) expected mode 0 to flutter!\n')
    if nipk.f_speed < 289. or  nipk.f_speed > 290.:
        raise RuntimeError('Test failed: (Nipk) expected flutter speed to be 289!\n')
    if nipk.f_mode != 0:
        raise RuntimeError('Test failed: (Pk) expected mode 0 to flutter!\n')
    if nipk.f_speed < 289. or  nipk.f_speed > 290.:
        raise RuntimeError('Test failed: (Pk) expected flutter speed to be 289!\n')
    print('All tests ok.')

if __name__ == '__main__':
    main()
